/*
 * Copyright 2009 Arne Vajh�j.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *    http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License. 
 */

package dk.vajhoej.isam.test;

import static org.junit.Assert.*;

import java.io.StringWriter;
import java.util.logging.Formatter;
import java.util.logging.Handler;
import java.util.logging.Level;
import java.util.logging.LogRecord;
import java.util.logging.Logger;

import dk.vajhoej.isam.IsamException;
import dk.vajhoej.isam.IsamResult;
import dk.vajhoej.isam.IsamSource;
import dk.vajhoej.isam.Key;
import dk.vajhoej.isam.Key0;
import dk.vajhoej.isam.cache.CacheIsamSource;
import dk.vajhoej.record.RecordException;

public class TestCommonIsamSource {
	private final static int N = 1000; //100000;
	private final static int N2 = 100;
	public static void testCreateRead(IsamSource isam) {
		try {
			Data o = new Data();
			o.setIv(258);
			o.setXv(123.456);
			o.setSv("ABC     ");
			isam.create(o);
			Data o2 = isam.read(Data.class, new Key<Integer>(0, 258));
			assertEquals("iv", 258, o2.getIv());
			assertEquals("xv", 123.456, o2.getXv(), 0.0005);
			assertEquals("sv", "ABC     ", o2.getSv());
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
    public static void testCreateReadKey0(IsamSource isam) {
        try {
            Data o = new Data();
            o.setIv(258);
            o.setXv(123.456);
            o.setSv("ABC     ");
            isam.create(o);
            Data o2 = isam.read(Data.class, new Key0<Integer>(258));
            assertEquals("iv", 258, o2.getIv());
            assertEquals("xv", 123.456, o2.getXv(), 0.0005);
            assertEquals("sv", "ABC     ", o2.getSv());
            isam.close();
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
	public static void testCreateReadMulti(IsamSource isam) {
		try {
			for(int i = 0; i < 10; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			for(int i = 0; i < 10; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
				assertEquals("xv", 123.456, o2.getXv(), 0.0005);
				assertEquals("sv", "ABC     ", o2.getSv());
			}
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testReadPerf(IsamSource isam) {
		try {
			for(int i = 0; i < N; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			long t1 = System.currentTimeMillis();
			for(int i = 0; i < N; i++) {
				Data o2 = isam.read(Data.class, new Key<Integer>(0, 258 + i));
				assertEquals("iv", 258 + i, o2.getIv());
			}
			long t2 = System.currentTimeMillis();
			isam.close();
			assertTrue("time", t2-t1 < 5*N);
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testCreatePerf(IsamSource isam) {
		try {
			long t1 = System.currentTimeMillis();
			for(int i = 0; i < N; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			long t2 = System.currentTimeMillis();
			isam.close();
			assertTrue("time", t2-t1 < 10*N);
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testReadGE(IsamSource isam) {
		try {
			for(int i = 0; i < N2; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			int i = N2 - N2/3;
			IsamResult<Data> res = isam.readGE(Data.class, new Key<Integer>(0, 258 + i));
			while(res.read()) {
				Data o2 = res.current();
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("GE records", N2, i);
			res.close();
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testReadGT(IsamSource isam) {
		try {
			for(int i = 0; i < N2; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			int i = N2 - N2/3;
			IsamResult<Data> res = isam.readGT(Data.class, new Key<Integer>(0, 258 + i));
			i++;
			while(res.read()) {
				Data o2 = res.current();
				assertEquals("iv", 258 + i, o2.getIv());
				i++;
			}
			assertEquals("GT records", N2, i);
			res.close();
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testReadLE(IsamSource isam) {
		try {
			for(int i = 0; i < N2; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			int i = N2 - N2/3;
			IsamResult<Data> res = isam.readLE(Data.class, new Key<Integer>(0, 258 + i));
			while(res.read()) {
				Data o2 = res.current();
				assertEquals("iv", 258 + i, o2.getIv());
				i--;
			}
			assertEquals("LE records", -1, i);
			res.close();
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testReadLT(IsamSource isam) {
		try {
			for(int i = 0; i < N2; i++) {
				Data o = new Data();
				o.setIv(258 + i);
				o.setXv(123.456);
				o.setSv("ABC     ");
				isam.create(o);
			}
			int i = N2 - N2/3;
			IsamResult<Data> res = isam.readLT(Data.class, new Key<Integer>(0, 258 + i));
			i--;
			while(res.read()) {
				Data o2 = res.current();
				assertEquals("iv", 258 + i, o2.getIv());
				i--;
			}
			assertEquals("LT records", -1, i);
			res.close();
			isam.close();
		} catch (IsamException e) {
			fail("Unexpected exception: " + e);
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testError1(IsamSource isam) {
		try {
			ErrorData1 o = new ErrorData1();
			o.setIv(123);
			isam.create(o);
			isam.close();
			fail("Missing exception");
		} catch (IsamException e) {
			try {
				isam.close();
			} catch (IsamException e1) {
				// ignore
			}
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
	public static void testError2(IsamSource isam) {
		try {
			ErrorData2 o = new ErrorData2();
			o.setXv(123.456);
			isam.create(o);
			isam.close();
			fail("Missing exception");
		} catch (IsamException e) {
			try {
				isam.close();
			} catch (IsamException e1) {
				// ignore
			}
		} catch (RecordException e) {
			fail("Unexpected exception: " + e);
		}
	}
    public static void testUpdate(IsamSource isam) {
        try {
            Data o = new Data();
            o.setIv(258);
            o.setXv(123.456);
            o.setSv("ABC     ");
            isam.create(o);
            Data o2 = isam.read(Data.class, new Key<Integer>(0, 258));
            assertEquals("iv", 258, o2.getIv());
            assertEquals("xv", 123.456, o2.getXv(), 0.0005);
            assertEquals("sv", "ABC     ", o2.getSv());
            o.setXv(123.789);
            o.setSv("ABCD    ");
            isam.update(o);
            o2 = isam.read(Data.class, new Key<Integer>(0, 258));
            assertEquals("iv", 258, o2.getIv());
            assertEquals("xv", 123.789, o2.getXv(), 0.0005);
            assertEquals("sv", "ABCD    ", o2.getSv());
            isam.close();
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    public static void testDelete(IsamSource isam) {
        try {
            Data o = new Data();
            o.setIv(258);
            o.setXv(123.456);
            o.setSv("ABC     ");
            isam.create(o);
            Data o2 = isam.read(Data.class, new Key<Integer>(0, 258));
            assertNotNull("o2", o2);
            isam.delete(Data.class, new Key<Integer>(0, 258));
            o2 = isam.read(Data.class, new Key<Integer>(0, 258));
            assertNull("o2", o2);
            isam.close();
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    public static void testLogging(IsamSource isam) {
        try {
            int n1 = testLogging(isam, Level.INFO);
            assertEquals("INFO log level", 0, n1);
            int n2 = testLogging(isam, Level.FINE);
            assertTrue("FINE log level", n2 > n1);
            int n3 = testLogging(isam, Level.FINER);
            assertTrue("FINER log level", n3 > n2);
            int n4 = testLogging(isam, Level.FINEST);
            assertTrue("FINEST log level", n4 > n3);
            isam.close();
        } catch (IsamException e) {
            e.printStackTrace();
            fail("Unexpected exception: " + e);
        }
    }
    public static int testLogging(IsamSource isam, Level lvl) {
        final StringWriter lw = new StringWriter();
        Logger log = Logger.getLogger("dk.vajhoej.isam");
        Level savlvl = log.getLevel();
        log.setLevel(lvl);
        Handler h = new Handler() {
            private Formatter fmt = new Formatter() {
                public String format(LogRecord record) {
                    return String.format("%1$tY-%1$tm-%1$td %1$tH:%1$tM:%1$tS %2$s %3$s: %4$s",
                            record.getMillis(), record.getSourceClassName(), record.getLevel(), record.getMessage());
                }
            };
            public void close() throws SecurityException {
            }
            public void flush() {
            }
            public void publish(LogRecord record) {
                lw.write(fmt.format(record) + "\r\n");
            }
        };
        log.addHandler(h);
        try {
            Data o = new Data();
            o.setIv(258);
            o.setXv(123.456);
            o.setSv("ABC     ");
            isam.create(o);
            Data o2 = isam.read(Data.class, new Key<Integer>(0, 258));
            o2.toString();
            isam.delete(Data.class, new Key<Integer>(0, 258));
        } catch (IsamException e) {
            e.printStackTrace();
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            e.printStackTrace();
            fail("Unexpected exception: " + e);
        }
        log.removeHandler(h);
        log.setLevel(savlvl);
        String logoutput = lw.getBuffer().toString();
        return logoutput.length();
    }
    public static void testConcurrency(IsamSource isam1, IsamSource isam2) {
        try {
            Data o = new Data();
            o.setIv(258);
            o.setXv(123.456);
            o.setSv("ABC     ");
            isam1.create(o);
            Data o2 = isam1.read(Data.class, new Key0<Integer>(258));
            assertNotNull("o2", o2);
            Data o3 = isam2.read(Data.class, new Key0<Integer>(258));
            assertNotNull("o3", o3);
            isam1.delete(Data.class, new Key0<Integer>(258));
            o2 = isam1.read(Data.class, new Key0<Integer>(258));
            assertTrue("o2", o2 == null);
            o3 = isam2.read(Data.class, new Key0<Integer>(258));
            assertTrue("o3", ((o3 == null) && !(isam2 instanceof CacheIsamSource)) || ((o3 != null) && (isam2 instanceof CacheIsamSource)));
            isam1.close();
            isam2.close();
        } catch (IsamException e) {
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            fail("Unexpected exception: " + e);
        }
    }
    public static void testConcurrencyHard(IsamSource isam1, IsamSource isam2) {
        try {
            for(int i = 0; i < N2; i++) {
                Data o = new Data();
                o.setIv(258 + i);
                o.setXv(123.456);
                o.setSv("ABC     ");
                isam1.create(o);
            }
            IsamResult<Data> res1 = isam1.readGE(Data.class, new Key0<Integer>(258));
            IsamResult<Data> res2 = isam2.readGE(Data.class, new Key0<Integer>(258));
            int i = 0;
            while(res1.read()) {
                Data o2 = res1.current();
                assertEquals("iv", 258 + i, o2.getIv());
                assertTrue("res2 read", res2.read());
                Data o3 = res2.current();
                assertEquals("iv", 258 + i, o3.getIv());
                i++;
            }
            res1.close();
            isam1.close();
            isam2.close();
        } catch (IsamException e) {
            e.printStackTrace();
            fail("Unexpected exception: " + e);
        } catch (RecordException e) {
            e.printStackTrace();
            fail("Unexpected exception: " + e);
        }
    }
}
